<?php
declare(strict_types=1);

namespace Intergo\LaravelAuthMiddleware\Http\Middleware;

use Intergo\LaravelAuthMiddleware\Models\AuthUser;
use Intergo\BaseClient\Responses\BaseResponse;
use Intergo\Log\GrayLogHandler;
use Intergo\LaravelAuthMiddleware\Services\Microservices\AuthService;
use Intergo\LaravelAuthMiddleware\Traits\ResponseTrait;
use Illuminate\Http\JsonResponse;
use Illuminate\Http\Request;
use Closure;

abstract class AbstractAuthMiddleware
{
    use ResponseTrait;
    /**
     * @var Request
     */
    protected $request;
    /**
     * @param Request $request
     * @param Closure $next
     * @param string $params
     * @return User|JsonResponse
     */
    public function handle(Request $request, Closure $next, string $params = '')
    {
        $this->setRequest($request);
        $result = $this->authenticateUserThroughAuth();
        if (!$result->isSuccessful()) {
            return null;
        }
        return $this->addUserToRequest($result->getData());
    }
    /**
     * @param Request $request
     */
    protected function setRequest(Request $request)
    {
        $this->request = $request;
    }
    /**
     * @return BaseResponse
     */
    protected function authenticateUserThroughAuth()
    {
        $token = $this->request->bearerToken() ?? $this->request->api_key;
        if (empty($token)) {
            $message = 'API Authenticate: Token is missing';
            $loggingHanlder = app(GrayLogHandler::class);
            $loggingTag = env('AUTH_TAG',env('APP_NAME','AuthService'));
            $loggingHanlder->setTag($loggingTag);
            $loggingHanlder->error($message);
            return new BaseResponse(['success' => false, 'message' => $message]);
        }
        //the default IP is in cases we can't determine the api from the request
        $ip = !empty($this->request->ip()) ? $this->request->ip() : "0.0.0.0";
        $authService = new AuthService();
        return $authService->authenticate($token, $ip, '');
    }
    /**
     * @param $user
     * @return User
     */
    protected function addUserToRequest($user)
    {
        $user['_id'] = $user['id'];
        $userCollection = AuthUser::hydrate(["user" => $user]);
        $user = $userCollection->first();
        $loggingHanlder = app(GrayLogHandler::class);
        $loggingTag = env('AUTH_TAG',env('APP_NAME','AuthService'));
        $loggingHanlder->setTag($loggingTag);
        $loggingHanlder->info('API Authenticated', compact('user'));
        \Auth::setUser($user); // This is setting the user to laravel Auth resolver, to be able to use it everywhere
        return $this->request->user = $user;
    }
}
